function __decorate(decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
}
var BinarySearch = require('binarysearch');
// @ts-ignore
var Levenshtein = require('damerau-levenshtein');

var SuggestRadius = 1000;
//Diccionario de datos
class Dictionary {
    /**
     * Creates an instance of Dictionary.
     *
     * @constructor
     * @param wordlist A sorted array of strings.
     */
    constructor(wordlist) {
        this.wordlist = [];
        this.setWordlist(wordlist);
        this.clearRegexs();
    }
    /**
     * Returns the number of words in the dictionary.
     *
     * @return {number} The number of words in the dictionary.
     */
    getLength() {
        return this.wordlist != null ? this.wordlist.length : 0;
    };
    /**
     * Set the list of words of the dictionary. a new Circle from a diameter.
     *
     * @param {string[]} wordlist A sorted array of strings.
     */
    setWordlist(wordlist) {
        if (wordlist != null && Array.isArray(wordlist)) {
            this.wordlist = wordlist;
        }
    };
    /**
     * Verify if a word is in the dictionary.
     *
     * @param {string} word A string.
     * @return {bool} 'true' if the word is in the dictionary, 'false' otherwise.
     */
    spellCheck(word) {
        // Verify if the word satifies one of the regular expressions.
        for (var i = 0; i < this.regexs.length; i++) {
            if (this.regexs[i].test(word)) {
                return true;
            }
        }
        // Since the list is sorted, is more fast to do a binary search than 'this.wordlist.indexOf(word)'.
        var res = BinarySearch(this.wordlist, // Haystack
            word.toLowerCase(), // Needle
            Collator.compare // Comparison method,
        );
        return res >= 0;
    };
    /**
     * Verify if a word is misspelled.
     *
     * @param {string} word A string.
     * @return {bool} 'true' if the word is misspelled, 'false' otherwise.
     */
    isMisspelled(word) {
        return !this.spellCheck(word);
    };
    /**
     * Get a list of suggestions for a misspelled word.
     *
     * @param {string} word A string.
     * @param {number} limit An integer indicating the maximum number of suggestions (by default 5).
     * @param {number} maxDistance An integer indicating the maximum edit distance between the word and the suggestions (by default 3).
     * @return {string[]} An array of strings with the suggestions.
     */
    getSuggestions(word, limit, maxDistance) {
        if (limit === void 0) { limit = 5; }
        if (maxDistance === void 0) { maxDistance = 3; }
        var suggestions = [];
        if (word != null && word.length > 0) {
            // Validate parameters.
            word = word.toLowerCase();
            if (limit == null || isNaN(limit) || limit <= 0) {
                limit = 5;
            }
            if (maxDistance == null || isNaN(maxDistance) || maxDistance <= 0) {
                maxDistance = 2;
            }
            if (maxDistance >= word.length) {
                maxDistance = word.length - 1;
            }
            // Search index of closest item.
            var closest = BinarySearch.closest(this.wordlist, word, Collator.compare);
            // Initialize variables for store results.
            var res = [];
            for (var i = 0; i <= maxDistance; i++) {
                res.push([]);
            }
            // Search suggestions around the position in which the word would be inserted.
            var k = void 0, dist = void 0;
            for (var i = 0; i < SuggestRadius; i++) {
                // The index 'k' is going to be 0, 1, -1, 2, -2...
                k = closest + (i % 2 != 0 ? ((i + 1) / 2) : (-i / 2));
                if (k >= 0 && k < this.wordlist.length) {
                    dist = Levenshtein(word, this.wordlist[k].toLowerCase()).steps;
                    if (dist <= maxDistance) {
                        res[dist].push(this.wordlist[k]);
                    }
                }
            }
            // Prepare result.
            for (var d = 0; d <= maxDistance && suggestions.length < limit; d++) {
                var remaining = limit - suggestions.length;
                suggestions = suggestions.concat((res[d].length > remaining) ? res[d].slice(0, remaining) : res[d]);
            }
        }
        return suggestions;
    };
    /**
     * Verify if a word is misspelled and get a list of suggestions.
     *
     * @param {string} word A string.
     * @param {number} limit An integer indicating the maximum number of suggestions (by default 5).
     * @param {number} maxDistance An integer indicating the maximum edit distance between the word and the suggestions (by default 3).
     * @return {Object} An object with the properties 'misspelled' (a boolean) and 'suggestions' (an array of strings).
     */
    checkAndSuggest(word, limit, maxDistance) {
        if (limit === void 0) { limit = 5; }
        if (maxDistance === void 0) { maxDistance = 3; }
        // Get suggestions.
        var suggestions = this.getSuggestions(word, limit + 1, maxDistance);
        // Prepare response.
        var res = { 'misspelled': true, 'suggestions': [] };
        res.misspelled = suggestions.length === 0 || suggestions[0].toLowerCase() !== word.toLowerCase();
        res.suggestions = suggestions;
        if (res.misspelled && (suggestions.length > limit)) {
            res.suggestions = suggestions.slice(0, limit);
        }
        if (!res.misspelled) {
            res.suggestions = suggestions.slice(1, suggestions.length);
        }
        // Verify if the word satifies one of the regular expressions.
        if (res.misspelled) {
            for (var i = 0; i < this.regexs.length; i++) {
                if (this.regexs[i].test(word)) {
                    res.misspelled = false;
                }
            }
        }
        return res;
    };
    /**
     * Adds a regular expression that will be used to verify if a word is valid even though is not on the dictionary.
     * Useful indicate that numbers, URLs and emails should not be marked as misspelled words.
     *
     * @param {RegEx} regexp A regular expression.
     */
    addRegex(regex) {
        this.regexs.push(regex);
    };
    /**
     * Clear the list of regultar expressions used to verify if a word is valid even though is not on the dictionary.
     */
    clearRegexs() {
        this.regexs = [];
    };

}


